FUNCTION_BLOCK Lcd_Write_Cmd
VAR CONSTANT

    DEF_I2C_BUS_TIMEOUT         : TIME              := t#1000ms;
    DEF_I2C_WRITE_DELAY         : TIME              := t#1ms;

END_VAR

VAR_INPUT

    I2cHandle_i                 : HI2C;
    bCommand_i                  : BYTE;
    bMode_i                     : BYTE;
    xEnable_i                   : BOOL;

END_VAR

VAR_OUTPUT

    usiError_o                  : USINT             := 0;
    xConfirm_o                  : BOOL              := FALSE;

END_VAR

VAR

    FB_I2cWrite                 : I2C_WRITE;

    bCommand                    : BYTE;
    abDataBlock                 : ARRAY[0..5] OF BYTE;
    pDataBlock                  : POINTER;

    xConfirm                    : BOOL;
    usiError                    : USINT;

    FB_RTrig                    : R_TRIG;
    iProcState                  : INT               := 0;
    FB_I2cWriteDelay            : TON;
END_VAR


    FB_RTrig (CLK := xEnable_i);
    IF (FB_RTrig.Q = TRUE) THEN
        iProcState := 0;
        usiError_o := 0;
        xConfirm_o := FALSE;
        FB_I2cWriteDelay (IN := FALSE);
    END_IF;

    IF (xEnable_i = FALSE) THEN
        RETURN;
    END_IF;


    CASE iProcState OF

        (* -------------------------------------------------------- *)
        (*                      H-Nibble                            *)
        (* -------------------------------------------------------- *)


    0:  (* ---- H-Nibble ---- *)
        bCommand := (bCommand_i AND 16#F0) OR bMode_i;
        abDataBlock[0] := bCommand OR LCD_BACKLIGHT;
        abDataBlock[1] := bCommand OR LCD_EN OR LCD_BACKLIGHT;
        abDataBlock[2] := (bCommand AND (LCD_EN XOR 16#FF)) OR LCD_BACKLIGHT;

        (* ---- L-Nibble ---- *)
        bCommand := (SHL(bCommand_i, 4) AND 16#F0) OR bMode_i;
        abDataBlock[3] := bCommand OR LCD_BACKLIGHT;
        abDataBlock[4] := bCommand OR LCD_EN OR LCD_BACKLIGHT;
        abDataBlock[5] := (bCommand AND (LCD_EN XOR 16#FF)) OR LCD_BACKLIGHT;

        pDataBlock := &abDataBlock;

        FB_I2cWrite (ENABLE := FALSE);
        iProcState := iProcState + 1;
        FB_I2cWriteDelay (IN := FALSE);

    1:
        FB_I2cWriteDelay (IN := TRUE, PT := DEF_I2C_WRITE_DELAY);
        IF (FB_I2cWriteDelay.Q = FALSE)
        THEN
            RETURN;
        END_IF;
        FB_I2cWrite (
            I2C_DEV_ADDR    := LCD_I2C_DEV_ADDRESS,
            MEM_ADDR        := 0,
            MEM_ADDR_TYPE   := I2C_NO_MEMADDR,
            PTR_WR_DATA     := pDataBlock,
            MAX_WR_LEN      := -1,
            TIMEOUT         := DEF_I2C_BUS_TIMEOUT,
            XFER_PEND       := FALSE,
            ENABLE          := TRUE,
            I2C_HANDLE      := I2cHandle_i
            |
            xConfirm        := CONFIRM,
            usiError        := ERROR);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;



        (* -------------------------------------------------------- *)
        (*                      Finish                              *)
        (* -------------------------------------------------------- *)

    2:  (* ---- Finish ---- *)

        usiError_o := 0;
        xConfirm_o := TRUE;
        RETURN;


        (* -------------------------------------------------------- *)
        (*                      Error                               *)
        (* -------------------------------------------------------- *)

    -1:  (* ---- Error ---- *)
        usiError_o := usiError;
        xConfirm_o := FALSE;
        RETURN;

    ELSE
        RETURN;

    END_CASE;



    RETURN;


END_FUNCTION_BLOCK